
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.012.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "Module to check if a country is in the European Union ( EU ) using various ISO data formats",
  "dist_author" => [
    "Matthew Green <matt\@mattsizzle.com>"
  ],
  "dist_name" => "Locale-Country-EU",
  "dist_version" => "0.004",
  "license" => "perl",
  "module_name" => "Locale::Country::EU",
  "recursive_test_files" => 1,
  "requires" => {
    "Carp" => 0,
    "Data::Util" => 0,
    "Exporter" => 0,
    "perl" => "5.016003"
  },
  "test_requires" => {
    "Test::More" => 0
  }
);


my %fallback_build_requires = (
  "Module::Build" => "0.28",
  "Test::More" => 0
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
